describe('General', function () {
  describe('nbQueued', function () {

    it('Should return the nbQueued with and without a priority value', function (done) {
      var c = makeTest(1, 250)

      console.assert(c.limiter.nbQueued() === 0)

      c.limiter.submit(c.job, null, 1, c.noErrVal(1))
      console.assert(c.limiter.nbQueued() === 0) // It's already running

      c.limiter.submit(c.job, null, 2, c.noErrVal(2))
      console.assert(c.limiter.nbQueued() === 1)
      console.assert(c.limiter.nbQueued(1) === 0)
      console.assert(c.limiter.nbQueued(5) === 1)

      c.limiter.submit(c.job, null, 3, c.noErrVal(3))
      console.assert(c.limiter.nbQueued() === 2)
      console.assert(c.limiter.nbQueued(1) === 0)
      console.assert(c.limiter.nbQueued(5) === 2)

      c.limiter.submit(c.job, null, 4, c.noErrVal(4))
      console.assert(c.limiter.nbQueued() === 3)
      console.assert(c.limiter.nbQueued(1) === 0)
      console.assert(c.limiter.nbQueued(5) === 3)

      c.limiter.submitPriority(1, c.job, null, 5, c.noErrVal(5))
      console.assert(c.limiter.nbQueued() === 4)
      console.assert(c.limiter.nbQueued(1) === 1)
      console.assert(c.limiter.nbQueued(5) === 3)

      c.last(function (err, results) {
        console.assert(c.limiter.nbQueued() === 0)
        c.checkResultsOrder([1,5,2,3,4])
        c.checkDuration(1000)
        console.assert(c.asserts() === 10)
        done()
      })
    })
  })

  describe('Events', function () {
    it('Should fire callback on empty queue', function (done) {
      var c = makeTest(1, 250)
      var called = 0

      c.limiter.on('empty', function () { called++ })

      c.pNoErrVal(c.limiter.schedule(c.promise, null, 1), 1)
      c.pNoErrVal(c.limiter.schedule(c.promise, null, 2), 2)
      c.pNoErrVal(c.limiter.schedule(c.promise, null, 3), 3)
      c.last(function (err, results) {
        c.checkResultsOrder([1,2,3])
        c.checkDuration(500)
        console.assert(c.asserts() === 3)
        console.assert(called === 2)
        done()
      })
    })

    it('Should fire events when calling stopAll()', function (done) {
      var c = makeTest(1, 250)
      var calledEmpty = 0
      var calledDropped = 0

      c.limiter.on('empty', function () { calledEmpty++ })
      c.limiter.on('dropped', function () { calledDropped++ })

      c.pNoErrVal(c.limiter.schedule(c.promise, null, 1), 1)
      c.pNoErrVal(c.limiter.schedule(c.promise, null, 2), 2)
      c.pNoErrVal(c.limiter.schedule(c.promise, null, 3), 3)

      c.limiter.stopAll()
      setTimeout(function () {
        console.assert(calledEmpty === 2)
        console.assert(calledDropped === 2)
        done()
      }, 20)
    })
  })

  describe('High water limit', function () {
    it('Should support highWater set to 0', function (done) {
      var c = makeTest(1, 250, 0)

      c.pNoErrVal(c.limiter.schedule(c.promise, null, 1), 1)
      c.pNoErrVal(c.limiter.schedule(c.promise, null, 2), 2)
      c.pNoErrVal(c.limiter.schedule(c.promise, null, 3), 3)
      c.pNoErrVal(c.limiter.schedule(c.promise, null, 4), 4)
      c.limiter.changeSettings(null, null, -1)
      c.last(function (err, results) {
        c.checkDuration(0)
        c.checkResultsOrder([1])
        console.assert(c.asserts() === 1)
        done()
      })
    })

    it('Should support highWater set to 1', function (done) {
      var c = makeTest(1, 250, 1)

      c.pNoErrVal(c.limiter.schedule(c.promise, null, 1), 1)
      c.pNoErrVal(c.limiter.schedule(c.promise, null, 2), 2)
      c.pNoErrVal(c.limiter.schedule(c.promise, null, 3), 3)
      c.pNoErrVal(c.limiter.schedule(c.promise, null, 4), 4)
      c.limiter.changeSettings(null, null, -1)
      c.last(function (err, results) {
        c.checkDuration(250)
        c.checkResultsOrder([1,4])
        console.assert(c.asserts() === 2)
        done()
      })
    })
  })
})